import { __decorate, __metadata } from 'tslib';
import { NgForm, Validators } from '@angular/forms';
import { Injectable, ElementRef, Input, Directive, NgModule } from '@angular/core';
import { debounceTime, filter, tap } from 'rxjs/operators';
import { Subject } from 'rxjs';
import tippy from 'tippy.js';

class ValidatorConfig {
    constructor(identifier, validator, message = "", tooltipsOnInit = false, alwaysEvaluate = false) {
        this.message = "";
        this.tooltipsOnInit = false;
        this.alwaysEvaluate = false;
        this.validator = validator;
        this.identifier = identifier;
        this.message = message,
            this.tooltipsOnInit = tooltipsOnInit;
        this.alwaysEvaluate = alwaysEvaluate;
    }
}

var ValidatorId;
(function (ValidatorId) {
    ValidatorId["Required"] = "required";
    ValidatorId["Pattern"] = "pattern";
    ValidatorId["RequiredTrue"] = "required";
    ValidatorId["NullValidator"] = "nullvalidator";
    ValidatorId["MinLength"] = "minlength";
    ValidatorId["MaxLength"] = "maxlength";
    ValidatorId["Email"] = "email";
    ValidatorId["Max"] = "max";
    ValidatorId["Min"] = "min";
    ValidatorId["Between"] = "between";
    ValidatorId["InvalidValues"] = "invalidValues";
    ValidatorId["RequiredValues"] = "requiredValues";
    ValidatorId["InvalidCharacters"] = "invalidCharacters";
    ValidatorId["SqlObjectName"] = "sqlObjectName";
    ValidatorId["IsInteger"] = "isInteger";
    ValidatorId["IsNumeric"] = "isNumeric";
    ValidatorId["isAlphaNumeric"] = "isAlphaNumeric";
    ValidatorId["isAlphabetical"] = "isAlphabetical";
    ValidatorId["startsWith"] = "startsWith";
    ValidatorId["endsWith"] = "endsWith";
})(ValidatorId || (ValidatorId = {}));

let NgEasyValidationService = class NgEasyValidationService {
    constructor() {
        this.validationResultSubject = new Subject();
    }
    validate(requirements, formGroup) {
        let result = [];
        for (let field in formGroup.controls) {
            const control = formGroup.controls[field];
            if (!control.errors)
                continue;
            for (let errorIdentifier in control.errors) {
                const config = JSON.parse(JSON.stringify(requirements[field].find(x => x.identifier == errorIdentifier)));
                this.setErrorMessage(field, control, config);
                result.push(new ValidationResult(field, config, !control.dirty && !config.tooltipsOnInit));
            }
        }
        ;
        this.validationResultSubject.next(result);
        return result;
    }
    setErrorMessage(field, control, config) {
        if (!config.message)
            config.message = this.generateErrorMessage(field, config.identifier, control.errors[config.identifier]);
        config.message = config.message.replace("<value>", control.value);
    }
    generateErrorMessage(field, requirementType, requirementDetails = null) {
        let errorMessage = "";
        field = field[0].toUpperCase() + field.substr(1);
        switch (requirementType) {
            case ValidatorId.Required:
                errorMessage = `${field} is required`;
                break;
            case ValidatorId.Pattern:
                errorMessage = `${field} isn't in the right format. The format must be: ${requirementDetails.requiredPattern}`;
                break;
            case ValidatorId.Between:
                errorMessage = `${field} must be between ${requirementDetails.min} and ${requirementDetails.max}`;
                break;
            case ValidatorId.Email:
                errorMessage = `${field} must be a valid e-mail address`;
                break;
            case ValidatorId.Max:
                errorMessage = `${field} must be lower than or equal to ${requirementDetails.max}`;
                break;
            case ValidatorId.Min:
                errorMessage = `${field} must be higher than or equal to ${requirementDetails.min}`;
                break;
            case ValidatorId.RequiredTrue:
                errorMessage = `${field} must be selected`;
                break;
            case ValidatorId.MinLength:
                errorMessage = `${field} has a minimum length of ${requirementDetails.requiredLength}`;
                break;
            case ValidatorId.MaxLength:
                errorMessage = `${field} has a maximum length of ${requirementDetails.requiredLength}`;
                break;
            case ValidatorId.RequiredValues:
                let msg = this.humanizeList(requirementDetails.requiredValues);
                errorMessage = `${field} must be ${msg}`;
                break;
            case ValidatorId.InvalidValues:
                let msg2 = this.humanizeList(requirementDetails.invalidValues);
                errorMessage = `${field} can't be ${msg2}`;
                break;
            case ValidatorId.InvalidCharacters:
                let msg3 = this.humanizeList(requirementDetails.invalidCharacters);
                errorMessage = `${field} can't contain ${msg3}`;
                break;
            case ValidatorId.SqlObjectName:
                errorMessage = `${field} isn't a valid SQL name`;
                break;
            case ValidatorId.IsInteger:
                errorMessage = `${field} must be an integer`;
                break;
            case ValidatorId.IsNumeric:
                errorMessage = `${field} must be a numeric`;
                break;
            case ValidatorId.isAlphaNumeric:
                errorMessage = `${field} must be alpha numerical`;
                break;
            case ValidatorId.isAlphabetical:
                errorMessage = `${field} can only contain letters`;
                break;
            case ValidatorId.startsWith:
                errorMessage = `${field} must start with '${requirementDetails.startsWith}'`;
                break;
            case ValidatorId.endsWith:
                errorMessage = `${field} must end with '${requirementDetails.endsWith}'`;
                break;
            default:
                errorMessage = `${field} is invalid`;
                break;
        }
        return errorMessage;
    }
    humanizeList(values) {
        let msg = "";
        for (let i = 0; i < values.length; i++) {
            if (i == values.length - 1)
                msg += " or ";
            msg += "'" + values[i] + "'";
            if (i < values.length - 2)
                msg += ", ";
        }
        return msg;
    }
};
NgEasyValidationService = __decorate([
    Injectable()
], NgEasyValidationService);
class ValidationResult {
    constructor(field, validatorConfig, isHidden) {
        this.field = field;
        this.validatorConfig = validatorConfig;
        this.isHidden = isHidden;
    }
}

let NgEasyValidationDirective = class NgEasyValidationDirective {
    constructor(el, validationService, host) {
        this.validationService = validationService;
        this.host = host;
        this.requirements = {};
        this.tippyProps = {};
        this.validationDebounceTime = 0;
        this.invalidClass = "validation-error";
        this.validationEnabled = false;
        this.tippies = {};
        this.formId = Math.random().toString(36).substring(2);
        el.nativeElement.setAttribute('formId', this.formId);
    }
    set ngEasyValidation(requirements) {
        let newRequirements = {};
        for (let field in requirements) {
            let configs = [];
            //Convert passed configurations to ValidatorConfig if they are not already
            for (let config of requirements[field]) {
                configs.push(new ValidatorConfig(config.validator.id, config.validator.validator, config.message, config.tooltipsOnInit || false, config.validator.alwaysEvaluate || false));
            }
            newRequirements[field] = configs;
        }
        this.requirements = newRequirements;
    }
    ngOnInit() {
        this.initValidationSubscription();
        this.setDefaultTippyProps();
    }
    initValidationSubscription() {
        this.validationSubscription = this.host.form.valueChanges
            .pipe(debounceTime(this.validationDebounceTime), filter(_ => this.validationEnabled), tap(this.validate.bind(this))).subscribe();
    }
    setDefaultTippyProps() {
        if (!this.tippyProps.allowHTML)
            this.tippyProps.allowHTML = true;
        tippy.setDefaultProps(this.tippyProps);
    }
    ngOnDestroy() {
        if (this.validationSubscription)
            this.validationSubscription.unsubscribe();
    }
    ngOnChanges(changes) {
        if (Object.keys(this.requirements).length == 0)
            this.validationEnabled = true;
        if (changes.ngEasyValidation) {
            setTimeout(() => {
                this.ngEasyValidation = changes.ngEasyValidation.currentValue;
                this.initFieldConfigurations();
                this.host.form.updateValueAndValidity();
            }, 200);
        }
    }
    initFieldConfigurations() {
        if (!this.requirements)
            return;
        this.validationEnabled = false;
        let inputFields = this.getUsableInputFields(this.getFormElement());
        for (let element of inputFields) {
            let field = element.getAttribute("name");
            let control = this.host.form.controls[field];
            let reqs = this.requirements[field];
            let isDirty = this.setFieldIsDirty(reqs, control);
            this.setFieldValidators(reqs, control);
            this.setFieldTippy(reqs, field, element, isDirty);
            control.updateValueAndValidity();
        }
        this.validationEnabled = true;
    }
    setFieldIsDirty(requirements, control) {
        let hasValue = control.value && control.value.length > 0;
        let mustValidateOnInit = requirements && requirements.some(x => x.tooltipsOnInit);
        if (hasValue || mustValidateOnInit)
            control.markAsDirty();
        return hasValue || mustValidateOnInit;
    }
    setFieldValidators(requirements, control) {
        let reqsToAdd = [];
        reqsToAdd.push(...requirements.map(x => x.validator));
        control.clearValidators();
        control.setValidators(reqsToAdd);
    }
    setFieldTippy(requirements, field, element, isDirty) {
        let instance = this.tippies[field];
        if (!instance) {
            instance = tippy(element);
            this.tippies[field] = instance;
            if (!isDirty && !requirements.some(x => x.tooltipsOnInit))
                setTimeout(instance.disable, 10);
        }
    }
    validate() {
        let formEl = this.getFormElement();
        if (!formEl)
            return;
        let inputFields = this.getUsableInputFields(formEl);
        this.validationEnabled = false;
        this.reEvaluateFields(inputFields);
        setTimeout(() => this.validationEnabled = true, this.validationDebounceTime);
        let validationResult = this.validationService.validate(this.requirements, this.host.form);
        this.updateFieldsLayout(inputFields, validationResult);
    }
    reEvaluateFields(inputFields) {
        let evaluateForm = false;
        for (let element of inputFields) {
            let field = element.getAttribute("name");
            let control = this.host.form.controls[field];
            if (this.requirements[field].some(x => x.alwaysEvaluate)) {
                control.updateValueAndValidity();
                evaluateForm = true;
            }
        }
        if (evaluateForm)
            this.host.form.updateValueAndValidity();
    }
    updateFieldsLayout(inputFields, allErrors) {
        for (let el of inputFields) {
            let field = el.getAttribute("name");
            let errors = allErrors.filter(x => x.field == field);
            let hasError = errors.length == 0 || errors.some(x => x.isHidden);
            this.updateFieldTippy(el, hasError, field, errors);
            this.updateFieldLayout(el, hasError);
        }
    }
    updateFieldTippy(el, hasError, field, errors) {
        let tippyInstance = this.tippies[field];
        if (hasError) {
            tippyInstance.disable();
        }
        else {
            let errorMessage = this.buildErrorMessage(errors);
            tippyInstance.setContent(errorMessage);
            tippyInstance.enable();
            if (this.isMouseOnElement(el))
                tippyInstance.show();
        }
    }
    updateFieldLayout(el, hasError) {
        let hasClass = el.classList.contains(this.invalidClass);
        if (hasError && !hasClass) {
            el.classList.add(this.invalidClass);
        }
        else if (!hasError && hasClass) {
            el.classList.remove(this.invalidClass);
        }
    }
    isMouseOnElement(element) {
        let hoverEls = document.querySelectorAll("input:hover, textarea:hover, select:hover");
        let hoverEl = hoverEls[hoverEls.length - 1];
        return hoverEl == element && element == document.activeElement;
    }
    buildErrorMessage(validationResults) {
        let errorMessage = "";
        let errorMessages = validationResults.map(x => x.validatorConfig.message || "This field is incorrect") || [];
        if (errorMessages.length > 1) {
            errorMessage += "<ul style='list-style-position:outside;padding-left:18px;margin:0'>";
            for (let msg of errorMessages) {
                errorMessage += "<li>" + msg + "</li>";
            }
            errorMessage += "</ul>";
        }
        if (errorMessages.length == 1) {
            errorMessage = errorMessages[0];
        }
        return errorMessage;
    }
    getFormElement() {
        return document.querySelectorAll('[formId="' + this.formId + '"]')[0];
    }
    getUsableInputFields(formElement) {
        let inputFields = formElement.querySelectorAll("input, textarea, select");
        let filteredInputFields = Array.from(inputFields).filter(x => Object.keys(this.requirements).includes(x.getAttribute("name")));
        return filteredInputFields;
    }
};
NgEasyValidationDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgEasyValidationService },
    { type: NgForm }
];
__decorate([
    Input("ngEasyValidation"),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], NgEasyValidationDirective.prototype, "ngEasyValidation", null);
__decorate([
    Input(),
    __metadata("design:type", Object)
], NgEasyValidationDirective.prototype, "tippyProps", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], NgEasyValidationDirective.prototype, "validationDebounceTime", void 0);
NgEasyValidationDirective = __decorate([
    Directive({
        selector: '[ngEasyValidation]',
        providers: [NgEasyValidationService]
    }),
    __metadata("design:paramtypes", [ElementRef,
        NgEasyValidationService,
        NgForm])
], NgEasyValidationDirective);

class ValidatorInfo {
    constructor(validator, id, alwaysEvaluate = false) {
        this.validator = validator;
        this.id = id;
        this.alwaysEvaluate = alwaysEvaluate;
    }
}

//Wrapper methods around Angular's validators
function required() {
    return new ValidatorInfo(Validators.required, ValidatorId.Required);
}
function pattern(regex) {
    return new ValidatorInfo(Validators.pattern(regex), ValidatorId.Pattern);
}
function requiredTrue() {
    return new ValidatorInfo(Validators.requiredTrue, ValidatorId.Required);
}
function nullValidator() {
    return new ValidatorInfo(Validators.required, ValidatorId.NullValidator);
}
function minLength(minLength) {
    return new ValidatorInfo(Validators.minLength(minLength), ValidatorId.MinLength);
}
function maxLength(maxLength) {
    return new ValidatorInfo(Validators.maxLength(maxLength), ValidatorId.MaxLength);
}
function email() {
    return new ValidatorInfo(Validators.required, ValidatorId.Email);
}
function min(min) {
    return new ValidatorInfo(Validators.min(min), ValidatorId.Min);
}
function max(max) {
    return new ValidatorInfo(Validators.max(max), ValidatorId.Max);
}
//Custom validators
function between(min, max) {
    const res = (control) => {
        return control.value && (control.value > max || control.value < min) ? { 'between': { value: control.value, max: max, min: min } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.Between);
}
function invalidValues(invalidValues, valueModifier = x => x) {
    const res = (control) => {
        let controlValue = valueModifier(control.value);
        if (!(invalidValues instanceof Array))
            invalidValues = [invalidValues];
        return invalidValues.map(x => typeof x == "string" ? x.toLowerCase() : x).includes(typeof controlValue == "string" ? controlValue.toLowerCase() : controlValue) ? { 'invalidValues': { value: controlValue, invalidValues: invalidValues } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.InvalidValues);
}
function requiredValues(requiredValues, valueModifier = x => x) {
    const res = (control) => {
        let controlValue = valueModifier(control.value);
        if (!(requiredValues instanceof Array))
            requiredValues = [requiredValues];
        return !requiredValues.map(x => typeof x == "string" ? x.toLowerCase() : x).includes(typeof controlValue == "string" ? controlValue.toLowerCase() : controlValue) ? { 'requiredValues': { value: controlValue, requiredValues: requiredValues } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.RequiredValues);
}
function invalidCharacters(invalidCharacters) {
    let regex = "^[^";
    for (let i = 0; i < invalidCharacters.length; i++) {
        const c = invalidCharacters[i];
        if ("[\^$.|?*+()".includes(c))
            regex += "\\";
        regex += c;
    }
    regex += "]*$";
    const res = (control) => {
        return !(new RegExp(regex, "i")).test(control.value) ? { 'invalidCharacters': { value: control.value, invalidCharacters: invalidCharacters } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.InvalidCharacters);
}
function sqlObjectName() {
    const res = (control) => {
        return (control.value && !(new RegExp("^[_a-zA-Zß#][0-9a-zA-Zß_@$#]*$", "i")).test(control.value)) ? { 'sqlObjectName': { value: control.value } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.SqlObjectName);
}
function conditionalValidator(validator, condition) {
    const res = (control) => {
        if (!condition(control))
            return null;
        return validator.validator(control);
    };
    return new ValidatorInfo(res, validator.id, true);
}
function isInteger() {
    const res = (control) => {
        let value = control.value || "";
        return !(new RegExp("^[0-9]*$", "i")).test(value) ? { 'isInteger': { value: value } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.IsInteger);
}
function isNumeric() {
    const res = (control) => {
        let value = control.value || "";
        return !(new RegExp("^[0-9]*[.,]?[0-9]*$", "i")).test(value) ? { 'isNumeric': { value: value } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.IsNumeric);
}
function isAlphaNumeric() {
    const res = (control) => {
        let value = control.value || "";
        var code, i, len;
        for (i = 0, len = value.length; i < len; i++) {
            code = value.charCodeAt(i);
            if (!(code > 47 && code < 58) && // numeric (0-9)
                !(code > 64 && code < 91) && // upper alpha (A-Z)
                !(code > 96 && code < 123)) { // lower alpha (a-z)
                return { 'isAlphaNumeric': { value: value } };
            }
        }
        return null;
    };
    return new ValidatorInfo(res, ValidatorId.isAlphaNumeric);
}
function isAlphabetical() {
    const res = (control) => {
        let value = control.value || "";
        var code, i, len;
        for (i = 0, len = value.length; i < len; i++) {
            code = value.charCodeAt(i);
            if (!(code > 64 && code < 91) && // upper alpha (A-Z)
                !(code > 96 && code < 123)) { // lower alpha (a-z)
                return { 'isAlphabetical': { value: value } };
            }
        }
        return null;
    };
    return new ValidatorInfo(res, ValidatorId.isAlphabetical);
}
function startsWith(str) {
    const res = (control) => {
        let value = control.value || "";
        return !value || value.startsWith(str) ? null : { "startsWith": { value: value, startsWith: str } };
    };
    return new ValidatorInfo(res, ValidatorId.startsWith);
}
function endsWith(str) {
    const res = (control) => {
        let value = control.value || "";
        return !value || value.endsWith(str) ? null : { "endsWith": { value: value, endsWith: str } };
    };
    return new ValidatorInfo(res, ValidatorId.endsWith);
}

let NgEasyValidationModule = class NgEasyValidationModule {
};
NgEasyValidationModule = __decorate([
    NgModule({
        imports: [],
        declarations: [
            NgEasyValidationDirective
        ],
        providers: [
            NgEasyValidationService
        ],
        exports: [
            NgEasyValidationDirective
        ]
    })
], NgEasyValidationModule);

/*
 * Public API Surface of ng-easy-validation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgEasyValidationDirective, NgEasyValidationModule, ValidatorInfo, between, conditionalValidator, email, endsWith, invalidCharacters, invalidValues, isAlphaNumeric, isAlphabetical, isInteger, isNumeric, max, maxLength, min, minLength, nullValidator, pattern, required, requiredTrue, requiredValues, sqlObjectName, startsWith, NgEasyValidationService as ɵa, ValidatorId as ɵb };
//# sourceMappingURL=ng-easy-validation.js.map
